from scripts.bigdirs import bigdirs, process_bigdirs
from scripts.christen import christen
from scripts.dedupe import dedupe
from scripts.merge_dbs import merge_dbs
from scripts.merge_online_local import merge_online_local
from scripts.mining.nfb_ca import nfb_films
from scripts.mining.nouns import nouns
from scripts.mining.pushshift import pushshift_extract
from scripts.mining.reddit_self import parse_reddit_selftext
from scripts.move_list import move_list
from scripts.optimize_db import optimize_db
from scripts.redownload import redownload
from scripts.relmv import relmv
from scripts.scatter import scatter, scatter_usage
from scripts.streaming_tab_loader import streaming_tab_loader
